package com.qmth.cqb.paper.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;

import com.qmth.cqb.paper.dao.ExtractConfigRepo;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.model.ExamPaper;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;

/**
 * 
 * @author  	chenken
 * @date    	2017年4月14日 下午6:07:55
 * @company 	QMTH
 * @description ExtractConfigServiceImpl.java
 */
@Service("extractConfigService")
public class ExtractConfigServiceImpl implements ExtractConfigService {
	
	@Autowired
    private ExtractConfigRepo extractConfigRepo;
	
	@Autowired
	private PaperDetailRepo paperDetailRepo;

    @Autowired
    private PaperDetailUnitRepo paperDetailUnitRepo;
    
    @Autowired
    private PaperDetailUnitService paperDetailUnitService;
    
    @Autowired
    PaperService paperService;
    
    @Autowired
    PaperRepo paperRepo;
    
	@Override
	public ExtractConfig findConfig(ExtractConfig condition) {
		if(condition.getExamId()==null){
			return null;
		}
		if(StringUtils.isBlank(condition.getCourseCode())){
			return null;
		}
		ExtractConfig tempConfig =  extractConfigRepo.findOne(Example.of(condition));
        return tempConfig;
	}

	@Override
	public void saveExtractConfig(ExtractConfig extractConfig) {
		List<ExamPaper> examPapers = extractConfig.getExamPaperList();
		for(int i=0;i<examPapers.size();i++){
			ExamPaper examPaper = examPapers.get(i);
			Paper paper = examPaper.getPaper();
			paper = paperRepo.findOne(paper.getId());
			examPaper.setPaper(paper);
		}
		extractConfigRepo.save(extractConfig);
	}

	@Override
	public ExtractConfig findConfigById(String id) {
		if(StringUtils.isBlank(id)){
			return null;
		}
		return extractConfigRepo.findOne(id);
	}
	@Override
	public Map<String, String> makePaperByConfig(String extractConfigId) {
		Map<String, String> finishedPaperIdMap = new HashMap<String, String>();
		//取出调卷规则
		ExtractConfig extractConfig = this.findConfig(new ExtractConfig(extractConfigId));
		//获得抽取好的试卷
		Map<String,Paper> paperMap = this.getExamPaperByProbability(extractConfig.getExamPaperList());
        if(paperMap==null){
        	return null;
        }
        Iterator<String> paperMapIterator = paperMap.keySet().iterator();
        while(paperMapIterator.hasNext()){
        	String key = paperMapIterator.next();
        	//根据原有试卷重组新试卷
        	Paper newPaper = this.recombinationPaper(paperMap.get(paperMapIterator.next()), 
							        			extractConfig.getScrambling_the_question_order(), 
							        			extractConfig.getScrambling_the_option_order());
        	finishedPaperIdMap.put(key, newPaper.getId());
        }
        return finishedPaperIdMap;
	}
	
	/**
	 * 每个试卷类型取出一套试卷
	 * {
	 * 	 A:Paper,
	 *   B:Paper
	 * }
	 * A是试卷类型   Paper是A类型下选定的试卷
	 * @param examPaperList		
	 * @return
	 */
	private Map<String,Paper> getExamPaperByProbability(List<ExamPaper> examPaperList){
		Map<String,Paper> paperByTypeMap = new HashMap<String,Paper>();
		if(examPaperList == null || examPaperList.size() == 0) {
       	 	return null;  
        }
		Map<String,List<ExamPaper>> examPaperMap = new HashMap<String,List<ExamPaper>>();
		for(int i = 0;i<examPaperList.size();i++){
			ExamPaper examPaper = examPaperList.get(i);
			if(!examPaperMap.containsKey(examPaper.getGroupCode())){
				if(examPaper.getPaper()!=null){
					List<ExamPaper> epList = new ArrayList<ExamPaper>();
					epList.add(examPaper);
					examPaperMap.put(examPaper.getGroupCode(), epList);
				}
			}else{
				if(examPaper.getPaper()!=null){
					List<ExamPaper> epList = examPaperMap.get(examPaper.getGroupCode());
					epList.add(examPaper);
				}
			}
		}
		Set<String> keys = examPaperMap.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			Paper paper = this.getPaperByProbability(examPaperMap.get(key));
			paperByTypeMap.put(key,paper);
		}
        return paperByTypeMap;
	}
	/**
	 * 根据设定几率取出一套试卷
	 * @param examPaperList
	 * @return
	 */
	private Paper getPaperByProbability(List<ExamPaper> examPaperList){
		int sum = 0;  
        for (int i = 0;i<examPaperList.size();i++) {  
             sum += examPaperList.get(i).getWeight();  
        }
        // 从1开始  
        Integer rand = new Random().nextInt(sum) + 1;  
        for (int i = 0;i<examPaperList.size();i++) {
            rand -= examPaperList.get(i).getWeight();
            // 选中
            if (rand <= 0) {
                return examPaperList.get(i).getPaper();
            }
        } 
        return null;
	}
	/**
	 * 重组试卷，生成新的试卷
	 * @param paper					选中的试卷
	 * @param upSetQuestionOrder	小题乱序		1:乱序  0：不乱序
	 * @param upSetOptionOrder		选项乱序		1:乱序  0：不乱序
	 * @return
	 */
	public Paper recombinationPaper(Paper paper,int upSetQuestionOrder,int upSetOptionOrder){
		paper.setId(null);
		Paper newPaper = paper;
		newPaper = paperRepo.save(newPaper);//重新保存paper
		List<PaperDetail> paperDetails = paperDetailRepo.findByPaper(paper);
		for(int i = 0;i<paperDetails.size();i++){
			PaperDetail paperDetail = paperDetails.get(i);
			paperDetail.setPaper(newPaper);//关联新Paper
			List<PaperDetailUnit> paperDetailUnits = paperDetailUnitService.getUnitsByPaperDetail(paperDetail);
			//小题乱序
			if(upSetQuestionOrder==1){
				Collections.shuffle(paperDetailUnits);
				for(int j = 0;i<paperDetailUnits.size();j++){
					PaperDetailUnit paperDetailUnit = paperDetailUnits.get(j);
					paperDetailUnit.setPaper(newPaper);//关联新Paper
					paperDetailUnit.setPaperDetail(paperDetail);//关联新paperDetail
					paperDetailUnit.setNumber(j+1);//重新设置序号
					//选项乱序
					if(upSetOptionOrder==1){
						//如果题目类型为单选或多选
						Question question = paperDetailUnit.getQuestion();
						QuesStructType quesStructType = question.getQuestionType();
						if(quesStructType==QuesStructType.SINGLE_ANSWER_QUESTION||
						   quesStructType==QuesStructType.MULTIPLE_ANSWER_QUESTION){
							List<QuesOption> options = question.getQuesOptions();
							Collections.shuffle(options);
							for(int k = 0;k<options.size();k++){
								QuesOption quesOption = options.get(k);
								Character num = (char) (0+64+1);
								quesOption.setNumber(String.valueOf(num));
							}
						}
					}
				}
			}
		}
		return newPaper;
	}
}
